<?php


class pageapi_LocalTaskCache {

  protected $_urlToWeight;
  protected $_routeToUrl;
  protected $_hasModifications = FALSE;

  function urlToWeight($url) {
    $this->_init();
    return $this->_urlToWeight[$url];
  }

  function moveLocalTask($route, $weight) {
    $this->_init();
    $url = $this->_routeToUrl($route);
    if (isset($this->_urlToWeight[$url]) && $this->_urlToWeight[$url] !== $weight) {
      $this->_urlToWeight[$url] = $weight;
      $this->_hasModifications = TRUE;
    }
  }

  function sortLocalTasks(&$html) {
    if ($this->_hasModifications) {
      $doc = new DOMDocument('1.0', 'iso-8859-1');
      $doc->loadXML($html);
      $this->_sortDOMTree($doc);
      $html = $doc->saveXML();
    }
  }

  function init() {
    $this->_init();
  }

  protected function _routeToUrl($route) {
    $this->_init();
    return $this->_routeToUrl[$route];
  }

  protected function _init() {

    if (!isset($this->_urlToWeight)) {
      $this->_urlToWeight = array();
      $this->_routeToUrl = array();

      $map = arg();
      $router_item = menu_get_item();
      if ($router_item && $router_item['access']) {
        // Get all tabs and the root page.
        $q = db_query("SELECT * FROM {menu_router} WHERE tab_root = '%s' ORDER BY weight, title", $router_item['tab_root']);

        while ($item = db_fetch_array($q)) {
          _menu_translate($item, $map, TRUE);
          // Store the translated item for later use.
          $url = url($item['href']);
          $this->_urlToWeight[$url] = $item['weight'];
          $this->_routeToUrl[$item['path']] = $url;
        }
      }
    }
  }

  protected function _sortDOMTree($ele) {
    if ($ele->tagName == 'ul') {
      $sortme = array();
      while ($c = $ele->firstChild) {
        $ele->removeChild($c);
        if ($c->tagName == 'li') {
          $href = $this->_findLinkHref($c);
          $weight = $this->urlToWeight($href);
          if (!isset($weight)) {
            $sortme[0][] = $c;
          }
          elseif ($weight === FALSE) {
            // Element is to be removed. Do nothing.
          }
          elseif (is_numeric($weight)) {
            // We got a weight setting for this element.
            $sortme[$weight][] = $c;
          }
        }
      }
      ksort($sortme, SORT_NUMERIC);
      if (count($sortme) > 1) {
        foreach ($sortme as $children) {
          foreach ($children as $c) {
            $ele->appendChild($c);
          }
        }
        return TRUE;
      }
      else {
        // Only one or zero tabs left? Then remove this level altogether.
        return FALSE;
      }
    }
    else {
      // Search for ul elements deeper down the hierarchy.
      $has_any_tabs = FALSE;
      $has_empty_ul = FALSE;
      foreach ($ele->childNodes as $c) {
        if (!empty($c->tagName)) {
          $has_tabs = $this->_sortDOMTree($c);
          if ($has_tabs === FALSE) {
            $ele->removeChild($c);
            $has_empty_ul = TRUE;
          }
          elseif ($has_tabs === TRUE) {
            $has_any_tabs = TRUE;
          }
        }
      }
      if ($has_any_tabs) {
        return TRUE;
      }
      elseif ($has_empty_ul) {
        return FALSE;
      }
    }
  }

  protected function _findLinkHref($ele) {
    foreach ($ele->childNodes as $c) {
      if ($c->tagName == 'a') {
        if ($c->hasAttribute('href')) {
          return $c->getAttribute('href');
        }
      }
      elseif (!empty($c->tagName)) {
        $candidate = $this->_findLinkHref($c);
        if (isset($candidate)) {
          return $candidate;
        }
      }
    }
  }
}
